local L = LibStub("AceLocale-3.0"):GetLocale("Elephant")
local Elephant = LibStub("AceAddon-3.0"):GetAddon("Elephant")

function Elephant:OnInitialize()
	--[[ Registering database with defaults: cloning objects to avoid problems ]]
	Elephant.db = LibStub("AceDB-3.0"):New("ElephantDB", {
		profile = Elephant:clone(Elephant.defaultConf.savedconfdefaults)
	})
	Elephant.dbpc = LibStub("AceDB-3.0"):New("ElephantDBPerChar", {
		char = Elephant:clone(Elephant.defaultConf.savedpercharconfdefaults)
	})
	
	--[[ General frame ]]
	Elephant:SetTitleInfoMaxLog()
	
	--[[ Elephant button ]]
	if Elephant.db.profile.button == true then
		Elephant:CreateButton()
	end
	
	--[[ Creating own slashcommands ]]
	local options = {
		type = 'group',
		args = {
			togglebutton = {
				type = 'toggle',
				name = L['showbutton'],
				desc = L['showbutton_desc'],
				get = function()
					return Elephant.db.profile.button
				end,
				set = function()
					Elephant:ToggleButton()
				end,
			},
			-- Deactivating prat options (see RegisterEventsRefresh() in Event.lua)
			--[[
			prat = {
				type = 'toggle',
				name = L['prat'],
				desc = L['prat_desc'],
				get = function()
					return Elephant.db.profile.prat
				end,
				set = function(_, v)
					Elephant.db.profile.prat = v
					Elephant:RegisterEventsRefresh()
				end,
				disabled = function()
					return not Prat
				end,
			},
			forcetimestamp = {
				type = 'toggle',
				name = L['forcetimestamp'],
				desc = L['forcetimestamp_desc'],
				get = function()
					return Elephant.db.profile.forcetimestamp
				end,
				set = function(_, v)
					Elephant.db.profile.forcetimestamp = v
				end,
				disabled = function()
					return (not Prat) or (not Elephant.db.profile.prat)
				end,
			},
			]]
			filters = {
				type = 'group',
				name = L['Filters'],
				desc = L['Filters_desc'],
				args = {
					add = {
						type = 'input',
						name = NEW,
						desc = L['newfilter_desc'],
						get = false,
						set = function(_, input)
							if string.match(input, L['filterregex']) == nil then
								Elephant:Print(format(L['filtererror'], input))
							else
								Elephant:AddFilter(input)
							end
						end,
						usage = L['filterusage'],
					},
					delete = {
						type = 'select',
						name = DELETE,
						desc = L['deletefilter_desc'],
						set = function(_,filterindex)
							Elephant:DeleteFilter(filterindex)
						end,
						values = function()
							return 	Elephant.db.profile.filters
						end,
						hidden = function()
							return (not Elephant.db.profile.filters) or
								(#Elephant.db.profile.filters == 0)
						end
					}
				},
			},
			log = {
				type = 'group',
				name = L['logs'],
				desc = L['logs_desc'],
				args = {
					clearall = {
						type = 'execute',
						name = L['clearallhelp'],
						desc = L['clearallhelp_desc'],
						func = function()
							StaticPopup_Show("ELEPHANT_CLEARALL")
						end,
					},
					default = {
						type = 'toggle',
						name = L['enableddefault'],
						desc = L['enableddefault_desc'],
						get = function()
							return Elephant.db.profile.defaultlog
						end,
						set = function(_, isEnabled)
							Elephant.db.profile.defaultlog = isEnabled
						end,
					},
					max = {
						type = 'range',
						name = L['maxlogwords'],
						desc = L['maxlogwords_desc'],
						min = Elephant.defaultConf.minlogsize,
						max = Elephant.defaultConf.maxlogsize,
						step = 1,
						get = function()
							return Elephant.db.profile.maxlog
						end,
						set = function(_, nb)
							Elephant:ChangeMaxLog(nb)
						end,
					},
					files = {
						type = 'group',
						name = L['files'],
						desc = L['files_desc'],
						args = {
							activate = {
								type = 'toggle',
								name = L['activate'],
								desc = L['activate_desc'],
								get = function()
									return Elephant.db.profile.activate_log
								end,
								set = function(_, nv)
									Elephant.db.profile.activate_log = nv
									Elephant:ChatLogEnable(Elephant.db.profile.chatlog)
									Elephant:CombatLogEnable(Elephant.db.profile.combatlog)
								end,
							},
							chat = {
								type = 'toggle',
								name = L['chatlog'],
								desc = L['chatlog_desc'],
								get = function()
									return Elephant.db.profile.chatlog
								end,
								set = function(_, isEnabled)
									Elephant.db.profile.chatlog = isEnabled
									Elephant:ChatLogEnable(isEnabled)
								end,
								hidden = function()
									return not Elephant.db.profile.activate_log
								end,
							},
							combat = {
								type = 'toggle',
								name = L['combatlog'],
								desc = L['combatlog_desc'],
								get = function()
									return Elephant.db.profile.combatlog
								end,
								set = function(_, isEnabled)
									Elephant.db.profile.combatlog = isEnabled
									Elephant:CombatLogEnable(isEnabled)
								end,
								hidden = function()
									return not Elephant.db.profile.activate_log
								end,
							},
						},
					},
				},
			},
			reset = {
				type = 'group',
				name = L['reset'],
				desc = L['reset_desc'],
				args = {
					all = {
						type = 'execute',
						name = L['resethelp'],
						desc = L['resethelp_desc'],
						func = function()
							StaticPopup_Show("ELEPHANT_RESET")
						end,
					},
					position = {
						type = 'execute',
						name = L['resetloc'],
						desc = L['resetloc_desc'],
						func = function()
							Elephant:ResetPosition()
						end,
					},
				},
			},
		}
	}
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Elephant", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Elephant")
	Elephant:RegisterChatCommand("elephant", function()
		InterfaceOptionsFrame_OpenToCategory("Elephant")
	end)
	
	--[[ Miscelleaneous elements status at loading ]]
	Elephant:ChatLogEnable(Elephant.db.profile.chatlog)
	Elephant:CombatLogEnable(Elephant.db.profile.combatlog)
	
	--[[ Checks & creates default log structures ]]
	Elephant:InitDefaultLogStructures()
	Elephant:AddHeaderToStructures()
end

function Elephant:OnEnable()
	--[[ Registering events ]]
	Elephant:RegisterEventsRefresh()
	
	--[[ Displays default log ]]
	if not Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex] then
		Elephant.dbpc.char.currentlogindex = Elephant.defaultConf.defaultlogindex
	end
	Elephant.tempConf.currentline = #Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].logs
	Elephant:ShowCurrentLog()
end